import { BrowserWindow, ipcMain, shell, app, BrowserView } from "electron";
import * as nodeMachine from "node-machine-id";
import * as path from "path";
import * as isDev from "electron-is-dev";
import { LoadURLOptions, UploadRawData } from "electron/main";
import { execFile } from "child_process";
import * as os from "os";

export default function registerIPC(
  mainWindow: BrowserWindow,
  userStore: any,
  serverStore: any
) {
  ipcMain.on("on-zoom-in", (event, arg) => {
    mainWindow.getBrowserView()?.webContents.setZoomLevel(3);
  });

  ipcMain.on("on-zoom-out", (event, arg) => {
    mainWindow.getBrowserView()?.webContents.setZoomLevel(0);
  });

  ipcMain.on("is-debug", (event, arg) => {
    let isDebug = !app.isPackaged;
    event.returnValue = isDebug;
  });

  ipcMain.on("get-version", (event, arg) => {
    let version = app.getVersion();
    event.returnValue = version;
  });

  ipcMain.on("get-machine-id", (event, arg) => {
    let id = nodeMachine.machineIdSync(true);
    event.returnValue = id;
  });

  ipcMain.on("get-machine-name", (event, arg) => {
    let id = os.hostname();
    event.returnValue = id;
  });

  ipcMain.on("get-user-store", (event, name) => {
    let data = userStore.get(name);
    event.returnValue = data;
  });

  ipcMain.on("set-user-store", (event, name, data) => {
    if (data && name) {
      userStore.set(name, data);
    }
    event.returnValue = true;
  });

  ipcMain.on("get-server-store", (event, name) => {
    let data = serverStore.get(name);
    event.returnValue = data;
  });

  ipcMain.on("set-server-store", (event, name, data) => {
    if (data && name) {
      serverStore.set(name, data);
    }
    event.returnValue = true;
  });

  ipcMain.on("get-path", (event, path) => {
    let data = app.getPath(path);
    event.returnValue = data;
  });

  ipcMain.on("path-join", (event, firstPath, secondPath) => {
    let data = path.join(firstPath, secondPath);
    event.returnValue = data;
  });

  ipcMain.on("path-dirname", (event, filepath) => {
    let data = path.dirname(filepath);
    event.returnValue = data;
  });

  ipcMain.on("get-path-log", (event, fileName) => {
    event.returnValue = getLogPath(fileName);
  });

  ipcMain.on("get-current-window-state", (e) => {
    console.log("get-current-window-state");

    e.returnValue = {
      isMaximized: BrowserWindow.getFocusedWindow()?.isMaximized() ?? false,
      isMinimizable: BrowserWindow.getFocusedWindow()?.isMinimizable() ?? false,
    };
  });

  ipcMain.on("change-current-window-state", (e, action) => {
    console.log("change-current-window-state: ", action);
    switch (action) {
      case "minimize": {
        BrowserWindow.getFocusedWindow()?.minimize();
        e.returnValue = null;
        return;
      }
      case "maximize": {
        BrowserWindow.getFocusedWindow()?.maximize();
        e.returnValue = null;
        return;
      }
      case "unmaximize": {
        BrowserWindow.getFocusedWindow()?.unmaximize();
        e.returnValue = null;
        return;
      }
      case "close": {
        BrowserWindow.getFocusedWindow()?.close();
        e.returnValue = null;
        return;
      }
      default: {
        e.returnValue = null;
      }
    }
  });

  ipcMain.on("open-web-page-on-browser", (event, link) => {
    shell.openExternal(link);
    event.returnValue = null;
  });

  ipcMain.on("open-fast-report", (event, link, accessToken, schema, body) => {
    // const win = new BrowserWindow({ width: 800, height: 600 });
    let frwin = new BrowserWindow({
      width: 800,
      height: 600,
      title: "Test",
    });
    frwin.setMenu(null);

    const view = new BrowserView();
    frwin.setBrowserView(view);
    view.setBounds({ x: 0, y: 0, width: 800, height: 600 });
    view.setAutoResize({
      width: true,
      height: true,
      horizontal: true,
      vertical: true,
    });

    // if (isDev) {
    //   // Open the DevTools.
    //   frwin.webContents.openDevTools();
    // }
    console.log("Access token: ", accessToken);
    console.log("Schema: ", schema);
    console.log("body: ", body);

    var loadOptions: LoadURLOptions;

    if (body) {
      let postData: UploadRawData = {
        bytes: Buffer.from(JSON.stringify(body)),
        type: "rawData",
      };

      loadOptions = {
        extraHeaders: `Authorization: Bearer ${accessToken}\n Content-Type: application/json\n charset=UTF-8\n`,
        postData: [postData],
      };
    } else {
      loadOptions = {
        extraHeaders: `Authorization: Bearer ${accessToken}\n Content-Type: application/json\n charset=UTF-8\n`,
      };
    }

    console.log("loadOptions: ", loadOptions);

    let frLink: string = `${link}?schema=${schema}`;
    view.webContents.loadURL(frLink, loadOptions);

    console.log("link: ", frLink);

    view.webContents.canGoBack = () => true;
    view.webContents.canGoForward = () => true;

    view.webContents.setWindowOpenHandler(({ url }) => {
      console.log("[open-fast-report] => setWindowOpenHandler: ", url);
      view.webContents.loadURL(url, loadOptions);

      if (url.includes("_fr")) return { action: "allow" };

      return { action: "deny" };
    });

    view.webContents.on(
      "before-input-event",
      (event: any, input: Electron.Input) => {
        if (input.control && input.key.toLowerCase() === "p") {
          // Hem d'imprimir
        } else if (input.key.toLowerCase() === "backspace") {
          view.webContents.goBack();
        }
      }
    );

    view.webContents.on(
      "did-navigate",
      async (
        _event: any,
        _url: string,
        httpResponseCode: number,
        httpStatusText: string
      ) => {
        console.log(
          "[open-fast-report] => did-navigate http code: ",
          httpResponseCode
        );
        console.log(
          "[open-fast-report] => did-navigate httpStatusText: ",
          httpStatusText
        );
        if (httpResponseCode >= 401) {
          // frwin.close();
          // console.log("refresh token and reload");
          // var r = await api.refreshAccessToken(
          //   api.accessToken,
          //   api.refreshToken
          // );
          // // what you want to do
          // const loadOptions = {
          //   extraHeaders: `Authorization: Bearer ${r.data?.accessToken}\n`,
          // };
          // let frLink: string = `${link}?schema=${api.schema}`;
          // view.webContents.loadURL(frLink, loadOptions);
        }
      }
    );

    event.returnValue = null;
  });
}

function getLogPath(filename: string) {
  if (isDev) {
    let logPath = path.join(
      path.dirname(app.getPath("logs")),
      `logs/${filename}`
    );
    return logPath;
  } else {
    let logPath = path.join(
      path.dirname(app.getPath("exe")),
      `logs/${filename}`
    );
    return logPath;
  }
}
